import types
import logging

from .utils import slurp


logger = logging.getLogger(__file__)


def list_branches_single_page(self, **kwargs):
    """
    This will continue to call list_branches until there are no more pages left to retrieve.  It will return
    the aggregated response in the same structure as list_branches does.

    :param self: codecommit client
    :param kwargs: these are passed onto the list_branches method call
    :return: codecommit_client.list_branches.response
    """
    return slurp(
        'list_branches',
        self.list_branches,
        'branches',
        'nextToken',
        'nextToken',
        **kwargs
    )


def make_better(client):
    client.list_branches_single_page = types.MethodType(list_branches_single_page, client)
    return client



import types
import logging
import time


logger = logging.getLogger(__file__)

put_parameter_and_wait_max_retries = 10


def put_parameter_and_wait(self, Name, **kwargs):
    """
    This will call put_parameter and ensure it has been successfully put by checking it

    :param self: ssm client
    :param Name: The fully qualified name of the parameter that you want to add to the system
    :param kwargs: These args are passed through to ssm.put_parameter
    :return: ssm.get_parameter.response
    """
    new_version = self.put_parameter(Name=Name, **kwargs).get('Version')
    count = 0
    current_parameter = {
        "Parameter": {
            "Version": -1
        }
    }
    while current_parameter.get('Parameter').get('Version') < new_version:
        count += 1
        if count > put_parameter_and_wait_max_retries:
            raise Exception(f"Putting and waiting for param {Name} failed")
        time.sleep(1)
        if new_version == 1:
            try:
                current_parameter = self.get_parameter(Name=Name)
            except self.exceptions.ParameterNotFound:
                pass
        else:
            current_parameter = self.get_parameter(Name=Name)
    return current_parameter


def get_parameter_history_single_page(self, **kwargs):
    """
    This will continue to call get_parameter_history_single_page until there are no more pages left to retrieve.
    It will return the aggregated response in the same structure as get_parameter_history_single_page does.

    :param self: ssm client
    :param kwargs: these are passed onto the get_parameter_history_single_page method call
    :return: ssm_client.get_parameter_history_single_page.response
    """
    return slurp(
        'get_parameter_history',
        self.get_parameter_history,
        'Parameters',
        'NextToken', 'NextToken',
        **kwargs
    )


class ParameterVersionNotFoundException(Exception):
    pass


def get_parameter_version(self, Version, **kwargs):
    paginator = self.get_paginator('get_parameter_history')
    iterator = paginator.paginate(
        **kwargs
    )
    for page in iterator:
        for parameter in page.get("Parameters", []):
            if parameter.get("Version") == Version:
                return dict(Parameter=parameter)

    raise ParameterVersionNotFoundException(f"Could not find version: {Version} of {kwargs.get('Name')}")


def make_better(client):
    client.put_parameter_and_wait = types.MethodType(put_parameter_and_wait, client)
    client.get_parameter_history_single_page = types.MethodType(get_parameter_history_single_page, client)
    client.get_parameter_version = types.MethodType(get_parameter_version, client)
    return client



import types
import logging

from .utils import slurp


logger = logging.getLogger(__file__)


def describe_budgets_single_page(self, **kwargs):
    """
    This will continue to call describe_budgets until there are no more pages left to retrieve.  It will return
    the aggregated response in the same structure as describe_budgets does.

    :param self: budgets client
    :param kwargs: these are passed onto the describe_budgets method call
    :return: budgets.describe_budgets.response
    """
    return slurp(
        'describe_budgets',
        self.describe_budgets,
        'Budgets',
        next_token_name_in_response='NextToken', next_token_name_in_request="NextToken",
        **kwargs
    )


def make_better(client):
    client.describe_budgets_single_page = types.MethodType(describe_budgets_single_page, client)
    return client



import types
import logging
import time


logger = logging.getLogger(__file__)


def start_build_and_wait_for_completion(self, **kwargs):
    """
    This will start a build of an AWS CodeBuild Project and wait for it to complete.
    It will return the result of the build.

    :param self: codebuild client
    :param kwargs: these are passed onto the start_build method call
    :return: codebuild_client.batch_get_builds.response[0]
    """
    build = self.start_build(
        **kwargs
    ).get('build')
    build_id = build.get('id')

    while build.get('buildStatus') == 'IN_PROGRESS':
        response = self.batch_get_builds(ids=[build_id])
        build = response.get('builds')[0]
        time.sleep(5)
        logger.info("Current status: {}".format(build.get('buildStatus')))

    return build


def make_better(client):
    client.start_build_and_wait_for_completion = types.MethodType(start_build_and_wait_for_completion, client)
    return client
